# -*- coding: utf8 -*-

from __future__ import division

import sys
import os
import shutil

import pyexiv2
from PIL import Image

EXTENSIONES_OK = "jpg JPG jpeg JPEG".split()

MIN_SIZE = (1024, 768)

USAGE = """
Usar  resize_imags.py <dir_fuente> <dir_destino>

    El programa toma todas las imágenes de la fuente, y las deja en el
    destino con el mismo nombre y tamaño más que %dx%d (rellenando a negro si
    es necesario).
""" % MIN_SIZE


def get_rot_image(path):
    # abrimos la imagen en PIL
    pil_img = Image.open(path)

    # rotamos en función de la info exif
    metadata = pyexiv2.ImageMetadata(path)
    metadata.read()
    if 'Exif.Image.Orientation' not in metadata.exif_keys:
        return pil_img

    orient = metadata['Exif.Image.Orientation'].value
    if orient == 1:
        # nada
        nueva = pil_img.copy()
    elif orient == 2:
        # espejo vertical
        nueva = pil_img.transpose(Image.FLIP_LEFT_RIGHT)
    elif orient == 3:
        # rotar 180°
        nueva = pil_img.transpose(Image.ROTATE_180)
    elif orient == 4:
        # espejo horizontal
        nueva = pil_img.transpose(Image.FLIP_TOP_BOTTOM)
    elif orient == 5:
        # espejo horizontal + rotado 270°
        nueva = pil_img.transpose(Image.FLIP_TOP_BOTTOM).transpose(Image.ROTATE_270)
    elif orient == 6:
        # rotado 270°
        nueva = pil_img.transpose(Image.ROTATE_270)
    elif orient == 7:
        # espejo vertical + rotado 270°
        nueva = pil_img.transpose(Image.FLIP_LEFT_RIGHT).transpose(Image.ROTATE_270)
    elif orient == 8:
        # rotado 90°
        nueva = pil_img.transpose(Image.ROTATE_90)
    else:
        print "WARNING: unknown orientation", orient
        nueva = pil_img.copy()

    return nueva


def get_new_image(oldimg, final_size):
    """Devuelve la imagen final a grabar."""
    oldx, oldy = oldimg.size
    print "  getting new image", (oldx, oldy)
    fforma = oldx / oldy
    if fforma == final_size[0] / final_size[1]:
        print u"  tenía el mismo factor de forma!"
        newimg = oldimg.resize(final_size, Image.ANTIALIAS)
        return newimg

    # armamos una imagen negra base
    negra = Image.new("RGB", final_size, (0,0,0))
    newy = final_size[1]

    if oldy > newy:
        # need to reduce it, resize original to make it fit in height
        newx = int(round(fforma * newy))
        print "  convertimos a %dx%d" % (newx, newy)
        newimg = oldimg.resize((newx, newy), Image.ANTIALIAS)

        # la centramos en la negra
        offset = int((final_size[0] - newx) / 2)
        negra.paste(newimg, (offset, 0))
    else:
        print "  too small, just center it"
        offset_x = int((final_size[0] - oldx) / 2)
        offset_y = int((final_size[1] - oldy) / 2)
        negra.paste(oldimg, (offset_x, offset_y))
    return negra


def main(dirsrc, dirdst):
    if not os.path.exists(dirdst):
        print "El dir %r no existe, lo creamos" % dirdst
        os.mkdir(dirdst)

    filessrc = os.listdir(dirsrc)

    # try to get a size for them all, choose smaller, ignore wrong orientation
    images_raw =  [(Image.open(os.path.join(dirsrc, x)), x) for x in filessrc]
    max_size_x = None
    for img, fname in images_raw:
        sizx, sizy = img.size
        if sizx < sizy:
            # wrong orientation
            continue
        if sizx < MIN_SIZE[0] or sizy < MIN_SIZE[1]:
            # too small
            continue
        if max_size_x is None or sizx < max_size_x:
            max_size_x = sizx
    size = (max_size_x, int(round(max_size_x * 3 / 4)))
    print "Selected size:", size


    for filename in filessrc:
        ext = filename.rsplit(".")[-1]
        if ext not in EXTENSIONES_OK:
            print "No procesamos %r por extensión incorrecta!" % filename
            continue

        print u"Trabajando", filename
        realpath = os.path.join(dirsrc, filename)
        im = get_rot_image(realpath)

        if im.size == size:
            print u"  ya era del tamaño final!"
            shutil.copy(realpath, dirdst)
        else:
            newim = get_new_image(im, size)
            newim.save(os.path.join(dirdst, filename))



if __name__ == "__main__":
    if len(sys.argv) != 3:
        print USAGE
        sys.exit()

    main(*sys.argv[1:])
